@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -76,6 +76,7 @@ struct GsAppPrivate
 	GHashTable		*urls;
 	gchar			*licence;
 	gchar			*menu_path;
+	gchar			*origin;
 	gchar			*update_version;
 	gchar			*update_version_ui;
 	gchar			*update_details;
@@ -91,6 +92,7 @@ struct GsAppPrivate
 	GdkPixbuf		*pixbuf;
 	GdkPixbuf		*featured_pixbuf;
 	GPtrArray		*related; /* of GsApp */
+	GHashTable		*related_hash; /* of "id-source" */
 	GPtrArray		*history; /* of GsApp */
 	guint64			 install_date;
 };
@@ -142,6 +144,10 @@ gs_app_kind_to_string (GsAppKind kind)
 		return "os-update";
 	if (kind == GS_APP_KIND_MISSING)
 		return "missing";
+	if (kind == GS_APP_KIND_SOURCE)
+		return "source";
+	if (kind == GS_APP_KIND_CORE)
+		return "core";
 	return NULL;
 }
 
@@ -265,6 +271,8 @@ gs_app_to_string (GsApp *app)
 		g_string_append_printf (str, "\tsummary-missing:\t%s\n", priv->summary_missing);
 	if (priv->menu_path != NULL && priv->menu_path[0] != '\0')
 		g_string_append_printf (str, "\tmenu-path:\t%s\n", priv->menu_path);
+	if (priv->origin != NULL && priv->origin[0] != '\0')
+		g_string_append_printf (str, "\torigin:\t%s\n", priv->origin);
 	if (priv->rating != -1)
 		g_string_append_printf (str, "\trating:\t%i\n", priv->rating);
 	if (priv->rating_confidence != -1)
@@ -515,6 +523,8 @@ gs_app_set_kind (GsApp *app, GsAppKind kind)
 		/* package can become either normal or a system application */
 		if (kind == GS_APP_KIND_NORMAL ||
 		    kind == GS_APP_KIND_SYSTEM ||
+		    kind == GS_APP_KIND_CORE ||
+		    kind == GS_APP_KIND_SOURCE ||
 		    kind == GS_APP_KIND_UNKNOWN)
 			state_change_ok = TRUE;
 		break;
@@ -526,6 +536,7 @@ gs_app_set_kind (GsApp *app, GsAppKind kind)
 		break;
 	case GS_APP_KIND_SYSTEM:
 	case GS_APP_KIND_OS_UPDATE:
+	case GS_APP_KIND_SOURCE:
 	case GS_APP_KIND_MISSING:
 		/* this can never change state */
 		break;
@@ -1127,6 +1138,30 @@ gs_app_set_menu_path (GsApp *app, const gchar *menu_path)
 	app->priv->menu_path = g_strdup (menu_path);
 }
 
+/**
+ * gs_app_get_origin:
+ */
+const gchar *
+gs_app_get_origin (GsApp *app)
+{
+	g_return_val_if_fail (GS_IS_APP (app), NULL);
+	return app->priv->origin;
+}
+
+/**
+ * gs_app_set_origin:
+ *
+ * The origin is the original source of the application to show in the UI,
+ * e.g. "Fedora"
+ */
+void
+gs_app_set_origin (GsApp *app, const gchar *origin)
+{
+	g_return_if_fail (GS_IS_APP (app));
+	g_free (app->priv->origin);
+	app->priv->origin = g_strdup (origin);
+}
+
 /**
  * gs_app_add_screenshot:
  */
@@ -1372,7 +1407,21 @@ gs_app_get_related (GsApp *app)
 void
 gs_app_add_related (GsApp *app, GsApp *app2)
 {
+	gchar *key;
+	gpointer found;
+
 	g_return_if_fail (GS_IS_APP (app));
+
+	key = g_strdup_printf ("%s-%s",
+			       gs_app_get_id_full (app2),
+			       gs_app_get_source_default (app2));
+	found = g_hash_table_lookup (app->priv->related_hash, key);
+	if (found != NULL) {
+		g_debug ("Already added %s as a related item", key);
+		g_free (key);
+		return;
+	}
+	g_hash_table_insert (app->priv->related_hash, key, GINT_TO_POINTER (1));
 	g_ptr_array_add (app->priv->related, g_object_ref (app2));
 }
 
@@ -1501,6 +1550,7 @@ gs_app_subsume (GsApp *app, GsApp *other)
 	const gchar *tmp;
 	GList *keys;
 	GList *l;
+	GsApp *app_tmp;
 	GsAppPrivate *priv2 = other->priv;
 	GsAppPrivate *priv = app->priv;
 	guint i;
@@ -1536,6 +1586,10 @@ gs_app_subsume (GsApp *app, GsApp *other)
 		tmp = g_ptr_array_index (priv2->categories, i);
 		gs_app_add_category (app, tmp);
 	}
+	for (i = 0; i < priv2->related->len; i++) {
+		app_tmp = g_ptr_array_index (priv2->related, i);
+		gs_app_add_related (app, app_tmp);
+	}
 
 	/* also metadata */
 	keys = g_hash_table_get_keys (priv2->metadata);
@@ -1746,6 +1800,10 @@ gs_app_init (GsApp *app)
 						     g_str_equal,
 						     g_free,
 						     g_free);
+	app->priv->related_hash = g_hash_table_new_full (g_str_hash,
+							 g_str_equal,
+							 g_free,
+							 NULL);
 	app->priv->urls = g_hash_table_new_full (g_str_hash,
 						 g_str_equal,
 						 g_free,
@@ -1769,6 +1827,7 @@ gs_app_finalize (GObject *object)
 	g_free (priv->icon);
 	g_free (priv->licence);
 	g_free (priv->menu_path);
+	g_free (priv->origin);
 	g_ptr_array_unref (priv->sources);
 	g_ptr_array_unref (priv->source_ids);
 	g_free (priv->project_group);
@@ -1783,6 +1842,7 @@ gs_app_finalize (GObject *object)
 	g_free (priv->update_details);
 	g_free (priv->management_plugin);
 	g_hash_table_unref (priv->metadata);
+	g_hash_table_unref (priv->related_hash);
 	g_ptr_array_unref (priv->related);
 	g_ptr_array_unref (priv->history);
 	if (priv->pixbuf != NULL)
