@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -57,11 +57,13 @@ typedef enum {
 
 typedef enum {
 	GS_APP_KIND_UNKNOWN,
-	GS_APP_KIND_NORMAL,	/* can be updated, removed and installed */
-	GS_APP_KIND_SYSTEM,	/* can be updated, but not installed or removed */
-	GS_APP_KIND_PACKAGE,	/* can be updated, but not installed or removed */
-	GS_APP_KIND_OS_UPDATE,	/* can be updated, but not installed or removed */
-	GS_APP_KIND_MISSING,	/* you can't do anything to this */
+	GS_APP_KIND_NORMAL,	/* app	[ install:1 remove:1 update:1 ] */
+	GS_APP_KIND_SYSTEM,	/* app	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_PACKAGE,	/* pkg	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_OS_UPDATE,	/* pkg	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_MISSING,	/* meta	[ install:0 remove:0 update:0 ] */
+	GS_APP_KIND_SOURCE,	/* src	[ install:1 remove:0 update:0 ] */
+	GS_APP_KIND_CORE,	/* pkg	[ install:0 remove:0 update:1 ] */
 	GS_APP_KIND_LAST
 } GsAppKind;
 
@@ -181,6 +183,9 @@ void		 gs_app_set_licence		(GsApp		*app,
 const gchar	*gs_app_get_menu_path		(GsApp		*app);
 void		 gs_app_set_menu_path		(GsApp		*app,
 						 const gchar	*menu_path);
+const gchar	*gs_app_get_origin		(GsApp		*app);
+void		 gs_app_set_origin		(GsApp		*app,
+						 const gchar	*origin);
 GPtrArray	*gs_app_get_screenshots		(GsApp		*app);
 void		 gs_app_add_screenshot		(GsApp		*app,
 						 GsScreenshot	*screenshot);
