@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -133,6 +133,10 @@ gs_cmd_refine_flag_from_string (const gchar *flag, GError **error)
 		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION;
 	if (g_strcmp0 (flag, "update-details") == 0)
 		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS;
+	if (g_strcmp0 (flag, "origin") == 0)
+		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN;
+	if (g_strcmp0 (flag, "related") == 0)
+		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_RELATED;
 	g_set_error (error,
 		     GS_PLUGIN_ERROR,
 		     GS_PLUGIN_ERROR_NOT_SUPPORTED,
@@ -303,6 +307,13 @@ main (int argc, char **argv)
 				break;
 			}
 		}
+	} else if (argc == 2 && g_strcmp0 (argv[1], "sources") == 0) {
+		list = gs_plugin_loader_get_sources (plugin_loader,
+						     refine_flags,
+						     NULL,
+						     &error);
+		if (list == NULL)
+			ret = FALSE;
 	} else if (argc == 2 && g_strcmp0 (argv[1], "popular") == 0) {
 		for (i = 0; i < repeat; i++) {
 			if (list != NULL)
@@ -371,7 +382,8 @@ main (int argc, char **argv)
 				     GS_PLUGIN_ERROR_FAILED,
 				     "Did not recognise option, use 'installed', "
 				     "'updates', 'popular', 'get-categories', "
-				     "'get-category-apps', 'filename-to-app', or 'search'");
+				     "'get-category-apps', 'filename-to-app', "
+				     "'sources', or 'search'");
 	}
 	if (!ret) {
 		g_print ("Failed: %s\n", error->message);
