@@ -227,6 +227,9 @@ gs_plugin_loader_run_refine (GsPluginLoader *plugin_loader,
 			     GError **error)
 {
 	GList *l;
+	GList *related_list = NULL;
+	GPtrArray *related;
+	GsApp *app;
 	GsPlugin *plugin;
 	gboolean ret = TRUE;
 	guint i;
@@ -253,6 +256,32 @@ gs_plugin_loader_run_refine (GsPluginLoader *plugin_loader,
 			goto out;
 	}
 
+	/* also do related packages one layer deep */
+	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_RELATED) > 0) {
+		flags &= ~GS_PLUGIN_REFINE_FLAGS_REQUIRE_RELATED;
+		for (l = *list; l != NULL; l = l->next) {
+			app = GS_APP (l->data);
+			related = gs_app_get_related (app);
+			for (i = 0; i < related->len; i++) {
+				app = g_ptr_array_index (related, i);
+				g_debug ("refining related: %s[%s]",
+					 gs_app_get_id (app),
+					 gs_app_get_source_default (app));
+				gs_plugin_add_app (&related_list, app);
+			}
+		}
+		if (related_list != NULL) {
+			ret = gs_plugin_loader_run_refine (plugin_loader,
+							   function_name_parent,
+							   &related_list,
+							   flags,
+							   cancellable,
+							   error);
+			if (!ret)
+				goto out;
+		}
+	}
+
 	/* now emit all the changed signals */
 	for (l = freeze_list; l != NULL; l = l->next)
 		g_object_thaw_notify (G_OBJECT (l->data));
@@ -260,6 +289,7 @@ gs_plugin_loader_run_refine (GsPluginLoader *plugin_loader,
 	/* dedupe applications we already know about */
 	gs_plugin_loader_list_dedupe (plugin_loader, *list);
 out:
+	gs_plugin_list_free (related_list);
 	gs_plugin_list_free (freeze_list);
 	return ret;
 }
@@ -432,6 +462,13 @@ gs_plugin_loader_app_is_valid (GsApp *app, gpointer user_data)
 		return FALSE;
 	}
 
+	/* don't show sources */
+	if (gs_app_get_kind (app) == GS_APP_KIND_SOURCE) {
+		g_debug ("app invalid as source %s",
+			 gs_plugin_loader_get_app_str (app));
+		return FALSE;
+	}
+
 	/* don't show unknown kind */
 	if (gs_app_get_kind (app) == GS_APP_KIND_UNKNOWN) {
 		g_debug ("app invalid as kind unknown %s",
@@ -896,6 +933,123 @@ gs_plugin_loader_get_updates_finish (GsPluginLoader *plugin_loader,
 
 /******************************************************************************/
 
+/**
+ * gs_plugin_loader_get_sources_thread_cb:
+ **/
+static void
+gs_plugin_loader_get_sources_thread_cb (GSimpleAsyncResult *res,
+					GObject *object,
+					GCancellable *cancellable)
+{
+	GError *error = NULL;
+	GsPluginLoaderAsyncState *state = (GsPluginLoaderAsyncState *) g_object_get_data (G_OBJECT (cancellable), "state");
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (object);
+
+	state->list = gs_plugin_loader_run_results (plugin_loader,
+						    "gs_plugin_add_sources",
+						    state->flags,
+						    cancellable,
+						    &error);
+	if (state->list == NULL) {
+		gs_plugin_loader_get_all_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* filter package list */
+	gs_plugin_list_filter_duplicates (&state->list);
+
+	/* dedupe applications we already know about */
+	gs_plugin_loader_list_dedupe (plugin_loader, state->list);
+
+	/* none left? */
+	if (state->list == NULL) {
+		g_set_error_literal (&error,
+				     GS_PLUGIN_LOADER_ERROR,
+				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS,
+				     "no sources to show");
+		gs_plugin_loader_get_all_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* success */
+	state->ret = TRUE;
+	gs_plugin_loader_get_all_state_finish (state, NULL);
+out:
+	return;
+}
+
+/**
+ * gs_plugin_loader_get_sources_async:
+ *
+ * This method calls all plugins that implement the gs_plugin_add_sources()
+ * function. The plugins return #GsApp objects of kind %GS_APP_KIND_SOURCE..
+ *
+ * The *applications* installed from each source can be obtained using
+ * gs_app_get_related() if this information is available.
+ **/
+void
+gs_plugin_loader_get_sources_async (GsPluginLoader *plugin_loader,
+				    GsPluginRefineFlags flags,
+				    GCancellable *cancellable,
+				    GAsyncReadyCallback callback,
+				    gpointer user_data)
+{
+	GCancellable *tmp;
+	GsPluginLoaderAsyncState *state;
+
+	g_return_if_fail (GS_IS_PLUGIN_LOADER (plugin_loader));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	/* save state */
+	state = g_slice_new0 (GsPluginLoaderAsyncState);
+	state->res = g_simple_async_result_new (G_OBJECT (plugin_loader),
+						callback,
+						user_data,
+						gs_plugin_loader_get_sources_async);
+	state->plugin_loader = g_object_ref (plugin_loader);
+	state->flags = flags;
+	if (cancellable != NULL)
+		state->cancellable = g_object_ref (cancellable);
+
+	/* run in a thread */
+	tmp = g_cancellable_new ();
+	g_object_set_data (G_OBJECT (tmp), "state", state);
+	g_simple_async_result_run_in_thread (G_SIMPLE_ASYNC_RESULT (state->res),
+					     gs_plugin_loader_get_sources_thread_cb,
+					     0,
+					     (GCancellable *) tmp);
+	g_object_unref (tmp);
+}
+
+/**
+ * gs_plugin_loader_get_sources_finish:
+ *
+ * Return value: (element-type GsApp) (transfer full): A list of applications
+ **/
+GList *
+gs_plugin_loader_get_sources_finish (GsPluginLoader *plugin_loader,
+				       GAsyncResult *res,
+				       GError **error)
+{
+	GSimpleAsyncResult *simple;
+
+	g_return_val_if_fail (GS_IS_PLUGIN_LOADER (plugin_loader), NULL);
+	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (res), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* failed */
+	simple = G_SIMPLE_ASYNC_RESULT (res);
+	if (g_simple_async_result_propagate_error (simple, error))
+		return NULL;
+
+	/* grab detail */
+	return gs_plugin_list_copy (g_simple_async_result_get_op_res_gpointer (simple));
+}
+
+/******************************************************************************/
+
 /**
  * gs_plugin_loader_get_installed_thread_cb:
  **/
@@ -1024,46 +1178,13 @@ gs_plugin_loader_get_installed_finish (GsPluginLoader *plugin_loader,
 
 /******************************************************************************/
 
-/**
- * gs_plugin_loader_get_popular_filter_cb:
- */
-static gboolean
-gs_plugin_loader_get_popular_filter_cb (GsApp *app, gpointer user_data)
-{
-	/* dont suggest already installed application */
-	if (gs_app_get_state (app) == GS_APP_STATE_INSTALLED)
-		return FALSE;
-
-	/* only show featured popular applications */
-	if (!gs_app_has_category (app, "featured"))
-		return FALSE;
-
-	/* require long description */
-	if (gs_app_get_description (app) == NULL)
-		return FALSE;
-
-	/* require at least one screenshot */
-	if (gs_app_get_screenshots (app)->len == 0)
-		return FALSE;
-
-	/* don't suggest applications we don't have a lot of confidence in,
-	 * just because one user rated the app 100% doesn't make it awesome
-	 * compared to an app with 300 votes of 85% */
-	if (gs_app_get_rating_confidence (app) < 90)
-		return FALSE;
-	if (gs_app_get_rating (app) < 75)
-		return FALSE;
-
-	return TRUE;
-}
-
 /**
  * gs_plugin_loader_get_popular_thread_cb:
  **/
 static void
 gs_plugin_loader_get_popular_thread_cb (GSimpleAsyncResult *res,
-					GObject *object,
-					GCancellable *cancellable)
+					  GObject *object,
+					  GCancellable *cancellable)
 {
 	GError *error = NULL;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (object);
@@ -1083,9 +1204,6 @@ gs_plugin_loader_get_popular_thread_cb (GSimpleAsyncResult *res,
 
 	/* filter package list */
 	gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_valid, NULL);
-	gs_plugin_list_filter (&state->list,
-			       gs_plugin_loader_get_popular_filter_cb,
-			       NULL);
 	if (state->list == NULL) {
 		g_set_error_literal (&error,
 				     GS_PLUGIN_LOADER_ERROR,
@@ -1129,7 +1247,7 @@ gs_plugin_loader_get_popular_async (GsPluginLoader *plugin_loader,
 						user_data,
 						gs_plugin_loader_get_popular_async);
 	state->plugin_loader = g_object_ref (plugin_loader);
-	state->flags = flags | GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING;
+	state->flags = flags;
 	if (cancellable != NULL)
 		state->cancellable = g_object_ref (cancellable);
 
@@ -1170,6 +1288,18 @@ gs_plugin_loader_get_popular_finish (GsPluginLoader *plugin_loader,
 
 /******************************************************************************/
 
+/**
+ * gs_plugin_loader_featured_debug:
+ **/
+static gboolean
+gs_plugin_loader_featured_debug (GsApp *app, gpointer user_data)
+{
+	if (g_strcmp0 (gs_app_get_id_full (app),
+	    g_getenv ("GNOME_SOFTWARE_FEATURED")) == 0)
+		return TRUE;
+	return FALSE;
+}
+
 /**
  * gs_plugin_loader_get_featured_thread_cb:
  **/
@@ -1195,8 +1325,12 @@ gs_plugin_loader_get_featured_thread_cb (GSimpleAsyncResult *res,
 	}
 
 	/* filter package list */
-	gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_valid, NULL);
-	gs_plugin_list_randomize (&state->list);
+	if (g_getenv ("GNOME_SOFTWARE_FEATURED") != NULL) {
+		gs_plugin_list_filter (&state->list, gs_plugin_loader_featured_debug, NULL);
+	} else {
+		gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_valid, NULL);
+		gs_plugin_list_randomize (&state->list);
+	}
 	if (state->list == NULL) {
 		g_set_error_literal (&error,
 				     GS_PLUGIN_LOADER_ERROR,
@@ -2488,7 +2622,6 @@ gs_plugin_loader_open_plugin (GsPluginLoader *plugin_loader,
 	plugin->pixbuf_size = 64;
 	plugin->priority = 0.f;
 	plugin->deps = plugin_deps != NULL ? plugin_deps (plugin) : NULL;
-	plugin->settings = g_object_ref (plugin_loader->priv->settings);
 	plugin->name = g_strdup (plugin_name ());
 	plugin->status_update_fn = gs_plugin_loader_status_update_cb;
 	plugin->status_update_user_data = plugin_loader;
@@ -2690,7 +2823,6 @@ gs_plugin_loader_plugin_free (GsPlugin *plugin)
 	g_free (plugin->name);
 	g_object_unref (plugin->profile);
 	g_hash_table_unref (plugin->icon_cache);
-	g_object_unref (plugin->settings);
 	g_module_close (plugin->module);
 	g_slice_free (GsPlugin, plugin);
 }
