@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -25,6 +25,8 @@
 
 #include "gs-plugin.h"
 
+#define GS_PLUGIN_OS_RELEASE_FN		"/etc/os-release"
+
 /**
  * gs_plugin_status_to_string:
  */
@@ -57,6 +59,38 @@ gs_plugin_set_enabled (GsPlugin *plugin, gboolean enabled)
 	plugin->enabled = enabled;
 }
 
+/**
+ * gs_plugin_check_distro_id:
+ **/
+gboolean
+gs_plugin_check_distro_id (GsPlugin *plugin, const gchar *distro_id)
+{
+	GError *error = NULL;
+	gboolean ret;
+	gchar *data = NULL;
+	gchar *search = NULL;
+
+	/* check that we are running on Fedora */
+	ret = g_file_get_contents (GS_PLUGIN_OS_RELEASE_FN,
+				   &data, NULL, &error);
+	if (!ret) {
+		g_warning ("%s could not be read: %s",
+			   GS_PLUGIN_OS_RELEASE_FN,
+			   error->message);
+		g_error_free (error);
+		goto out;
+	}
+	search = g_strdup_printf ("ID=%s\n", distro_id);
+	if (g_strstr_len (data, -1, search) == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+out:
+	g_free (data);
+	g_free (search);
+	return ret;
+}
+
 /**
  * gs_plugin_add_app:
  **/
@@ -105,7 +139,50 @@ gs_plugin_list_filter (GList **list, GsPluginListFilter func, gpointer user_data
 }
 
 /**
- * gs_plugin_list_copy:
+ * gs_plugin_list_randomize_cb:
+ */
+static gint
+gs_plugin_list_randomize_cb (gconstpointer a, gconstpointer b)
+{
+	const gchar *k1;
+	const gchar *k2;
+	k1 = gs_app_get_metadata_item (GS_APP (a), "Plugin::sort-key");
+	k2 = gs_app_get_metadata_item (GS_APP (b), "Plugin::sort-key");
+	return g_strcmp0 (k1, k2);
+}
+
+/**
+ * gs_plugin_list_randomize:
+ *
+ * Randomize the order of the list, but don't change the order until the next day
+ **/
+void
+gs_plugin_list_randomize (GList **list)
+{
+	GDateTime *date;
+	GList *l;
+	GRand *rand;
+	GsApp *app;
+	gchar sort_key[] = { '\0', '\0', '\0', '\0' };
+
+	rand = g_rand_new ();
+	date = g_date_time_new_now_utc ();
+	g_rand_set_seed (rand, g_date_time_get_day_of_year (date));
+	for (l = *list; l != NULL; l = l->next) {
+		app = GS_APP (l->data);
+		sort_key[0] = g_rand_int_range (rand, (gint32) 'A', (gint32) 'Z');
+		sort_key[1] = g_rand_int_range (rand, (gint32) 'A', (gint32) 'Z');
+		sort_key[2] = g_rand_int_range (rand, (gint32) 'A', (gint32) 'Z');
+		gs_app_set_metadata (app, "Plugin::sort-key", sort_key);
+	}
+	*list = g_list_sort (*list, gs_plugin_list_randomize_cb);
+
+	g_rand_free (rand);
+	g_date_time_unref (date);
+}
+
+/**
+ * gs_plugin_list_filter_duplicates:
  **/
 void
 gs_plugin_list_filter_duplicates (GList **list)
