@@ -72,7 +72,6 @@ struct GsPlugin {
 	gpointer		 updates_changed_user_data;
 	GsProfile		*profile;
 	GHashTable		*icon_cache;	/* key is the id.desktop */
-	GSettings		*settings;
 };
 
 typedef enum {
@@ -93,6 +92,8 @@ typedef enum {
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY		= 1 << 7,
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION	= 1 << 8,
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS	= 1 << 9,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN		= 1 << 10,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_RELATED		= 1 << 11,
 	GS_PLUGIN_REFINE_FLAGS_LAST
 } GsPluginRefineFlags;
 
@@ -179,6 +180,10 @@ gboolean	 gs_plugin_add_updates			(GsPlugin	*plugin,
 							 GList		**list,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_add_sources			(GsPlugin	*plugin,
+							 GList		**list,
+							 GCancellable	*cancellable,
+							 GError		**error);
 gboolean	 gs_plugin_add_updates_historical	(GsPlugin	*plugin,
 							 GList		**list,
 							 GCancellable	*cancellable,
