@@ -153,7 +153,8 @@ gs_shell_category_populate_filtered (GsShellCategory *shell)
 
 	gs_plugin_loader_get_category_apps_async (priv->plugin_loader,
 						  priv->category,
-						  GS_PLUGIN_REFINE_FLAGS_DEFAULT,
+						  GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+						  GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING,
 						  priv->cancellable,
 						  gs_shell_category_get_apps_cb,
 						  shell);
