@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -146,7 +146,7 @@ gs_shell_details_refresh (GsShellDetails *shell_details)
 	switch (state) {
 	case GS_APP_STATE_AVAILABLE:
 	case GS_APP_STATE_LOCAL:
-		gtk_widget_set_visible (widget, TRUE);
+		gtk_widget_set_visible (widget, gs_app_get_kind (priv->app) != GS_APP_KIND_CORE);
 		gtk_widget_set_sensitive (widget, TRUE);
 		gtk_style_context_add_class (gtk_widget_get_style_context (widget), "suggested-action");
 		/* TRANSLATORS: button text in the header when an application
@@ -348,6 +348,9 @@ gs_shell_details_refresh_screenshots (GsShellDetails *shell_details)
 						     "box_details_screenshot_main"));
 	gs_container_remove_all (GTK_CONTAINER (widget));
 	screenshots = gs_app_get_screenshots (priv->app);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+						     "box_details_screenshot"));
+	gtk_widget_set_visible (widget, screenshots->len > 0);
 	if (screenshots->len == 0) {
 		widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
 							     "box_details_screenshot_thumbnails"));
@@ -379,6 +382,8 @@ gs_shell_details_refresh_screenshots (GsShellDetails *shell_details)
 				      provided.width, provided.height);
 
 	gs_screenshot_image_load_async (GS_SCREENSHOT_IMAGE (ssimg), NULL);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+						     "box_details_screenshot_main"));
 	gtk_box_pack_start (GTK_BOX (widget), ssimg, FALSE, FALSE, 0);
 	gtk_widget_set_visible (ssimg, TRUE);
 
@@ -523,18 +528,6 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 	tmp = gs_app_get_description (priv->app);
 	gs_shell_details_set_description (shell_details, tmp);
 
-	/* do not show the 'Details' header if we are a local file and have no
-	 * long description */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "application_details_details_title"));
-	switch (gs_app_get_state (priv->app)) {
-	case GS_APP_STATE_LOCAL:
-		gtk_widget_set_visible (widget, tmp != NULL);
-		break;
-	default:
-		gtk_widget_set_visible (widget, TRUE);
-		break;
-	}
-
 	/* set the icon */
 	tmp = gs_app_get_metadata_item (priv->app, "DataDir::desktop-icon");
 	if (tmp != NULL) {
@@ -650,6 +643,24 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		gtk_label_set_label (GTK_LABEL (widget), tmp);
 	}
 
+	/* set the origin */
+	tmp = gs_app_get_origin (priv->app);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+						     "label_details_origin_value"));
+	if (tmp == NULL || tmp[0] == '\0') {
+		/* TRANSLATORS: this is where we don't know the origin of the
+		 * application */
+		gtk_label_set_label (GTK_LABEL (widget), _("Unknown"));
+	} else {
+		gtk_label_set_label (GTK_LABEL (widget), tmp);
+	}
+	gtk_widget_set_visible (widget,
+				gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+						     "label_details_origin_title"));
+	gtk_widget_set_visible (widget,
+				gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED);
+
 	/* set the rating */
 	switch (gs_app_get_id_kind (priv->app)) {
 	case GS_APP_ID_KIND_WEBAPP:
@@ -689,17 +700,6 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		break;
 	}
 
-	/* only show support box for non-local packages */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "box_details_support"));
-	switch (gs_app_get_state (priv->app)) {
-	case GS_APP_STATE_LOCAL:
-		gtk_widget_set_visible (widget, FALSE);
-		break;
-	default:
-		gtk_widget_set_visible (widget, TRUE);
-		break;
-	}
-
 	/* make history button insensitive if there is none */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_history"));
 	switch (gs_app_get_id_kind (priv->app)) {
@@ -712,6 +712,64 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		gtk_widget_set_visible (widget, TRUE);
 		break;
 	}
+
+	/* are we trying to replace something in the baseos */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_package_baseos"));
+	switch (gs_app_get_kind (priv->app)) {
+	case GS_APP_KIND_CORE:
+		gtk_widget_set_visible (widget, TRUE);
+		break;
+	default:
+		gtk_widget_set_visible (widget, FALSE);
+		break;
+	}
+
+	/* is this a repo-release */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_repo"));
+	switch (gs_app_get_kind (priv->app)) {
+	case GS_APP_KIND_SOURCE:
+		gtk_widget_set_visible (widget, gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
+		break;
+	default:
+		gtk_widget_set_visible (widget, FALSE);
+		break;
+	}
+
+	/* an application */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_repo"));
+	switch (gs_app_get_kind (priv->app)) {
+	case GS_APP_KIND_SOURCE:
+		gtk_widget_set_visible (widget, gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
+		break;
+	default:
+		gtk_widget_set_visible (widget, FALSE);
+		break;
+	}
+
+	/* installing a app with a repo file */
+	tmp = gs_app_get_metadata_item (priv->app, "PackageKit::has-source");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_app_repo"));
+	switch (gs_app_get_kind (priv->app)) {
+	case GS_APP_KIND_NORMAL:
+	case GS_APP_KIND_SYSTEM:
+		gtk_widget_set_visible (widget, tmp != NULL && gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
+		break;
+	default:
+		gtk_widget_set_visible (widget, FALSE);
+		break;
+	}
+
+	/* installing a app without a repo file */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_app_norepo"));
+	switch (gs_app_get_kind (priv->app)) {
+	case GS_APP_KIND_NORMAL:
+	case GS_APP_KIND_SYSTEM:
+		gtk_widget_set_visible (widget, tmp == NULL && gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
+		break;
+	default:
+		gtk_widget_set_visible (widget, FALSE);
+		break;
+	}
 }
 
 /**
@@ -830,6 +888,7 @@ gs_shell_details_set_app (GsShellDetails *shell_details, GsApp *app)
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION |
+					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_URL,
 					   priv->cancellable,
 					   gs_shell_details_app_refine_cb,
