@@ -33,10 +33,13 @@ struct AppstreamApp
 	GPtrArray		*pkgnames;
 	gint			 priority;
 	gchar			*name;
+	gchar			*name_lang;
 	guint			 name_value;
 	gchar			*summary;
+	gchar			*summary_lang;
 	guint			 summary_value;
 	gchar			*description;
+	gchar			*description_lang;
 	GHashTable		*urls;
 	gchar			*licence;
 	gchar			*project_group;
@@ -50,8 +53,27 @@ struct AppstreamApp
 	gpointer		 userdata;
 	GDestroyNotify		 userdata_destroy_func;
 	GPtrArray		*screenshots; /* of AppstreamScreenshot */
+	gsize			 token_cache_valid;
+	GPtrArray		*token_cache;
 };
 
+typedef struct {
+	gchar	**values_ascii;
+	gchar	**values_utf8;
+	guint	  score;
+} AppstreamAppTokenItem;
+
+/**
+ * appstream_app_token_item_free:
+ */
+static void
+appstream_app_token_item_free (AppstreamAppTokenItem *token_item)
+{
+	g_strfreev (token_item->values_ascii);
+	g_strfreev (token_item->values_utf8);
+	g_slice_free (AppstreamAppTokenItem, token_item);
+}
+
 /**
  * appstream_app_free:
  */
@@ -65,13 +87,17 @@ appstream_app_free (AppstreamApp *app)
 	g_free (app->project_group);
 	g_free (app->icon);
 	g_free (app->name);
+	g_free (app->name_lang);
 	g_free (app->summary);
+	g_free (app->summary_lang);
 	g_free (app->description);
+	g_free (app->description_lang);
 	g_ptr_array_unref (app->appcategories);
 	g_ptr_array_unref (app->keywords);
 	g_ptr_array_unref (app->mimetypes);
 	g_ptr_array_unref (app->desktop_core);
 	g_ptr_array_unref (app->screenshots);
+	g_ptr_array_unref (app->token_cache);
 	if (app->userdata_destroy_func != NULL)
 		app->userdata_destroy_func (app->userdata);
 	g_slice_free (AppstreamApp, app);
@@ -112,6 +138,7 @@ appstream_app_new (void)
 	app->pkgnames = g_ptr_array_new_with_free_func (g_free);
 	app->desktop_core = g_ptr_array_new_with_free_func (g_free);
 	app->screenshots = g_ptr_array_new_with_free_func ((GDestroyNotify) appstream_screenshot_free);
+	app->token_cache = g_ptr_array_new_with_free_func ((GDestroyNotify) appstream_app_token_item_free);
 	app->urls = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	app->name_value = G_MAXUINT;
 	app->summary_value = G_MAXUINT;
@@ -328,7 +355,9 @@ appstream_app_set_name (AppstreamApp *app,
 	new_value = appstream_get_locale_value (lang);
 	if (new_value < app->name_value) {
 		g_free (app->name);
+		g_free (app->name_lang);
 		app->name = g_strndup (name, length);
+		app->name_lang = g_strdup (lang);
 		app->name_value = new_value;
 	}
 }
@@ -347,7 +376,9 @@ appstream_app_set_summary (AppstreamApp *app,
 	new_value = appstream_get_locale_value (lang);
 	if (new_value < app->summary_value) {
 		g_free (app->summary);
+		g_free (app->summary_lang);
 		app->summary = g_strndup (summary, length);
+		app->summary_lang = g_strdup (lang);
 		app->summary_value = new_value;
 	}
 }
@@ -393,9 +424,11 @@ appstream_app_set_project_group (AppstreamApp *app,
  */
 void
 appstream_app_set_description (AppstreamApp *app,
+			       const gchar *lang,
 			       const gchar *description,
 			       gssize length)
 {
+	app->description_lang = g_strdup (lang);
 	if (length < 0)
 		app->description = g_strdup (description);
 	else
@@ -496,33 +529,81 @@ appstream_app_get_categories (AppstreamApp *app)
 }
 
 /**
- * appstream_app_search_matches:
+ * appstream_app_add_tokens:
  */
-gboolean
-appstream_app_search_matches (AppstreamApp *app, const gchar *search)
+static void
+appstream_app_add_tokens (AppstreamApp *app,
+			  const gchar *value,
+			  const gchar *lang,
+			  guint score)
+{
+	AppstreamAppTokenItem *token_item;
+
+	/* sanity check */
+	if (value == NULL)
+		return;
+
+	token_item = g_slice_new (AppstreamAppTokenItem);
+	token_item->values_utf8 = g_str_tokenize_and_fold (value,
+							   lang,
+							   &token_item->values_ascii);
+	token_item->score = score;
+	g_ptr_array_add (app->token_cache, token_item);
+}
+
+/**
+ * appstream_app_create_token_cache:
+ */
+static void
+appstream_app_create_token_cache (AppstreamApp *app)
 {
 	const gchar *tmp;
 	guint i;
 
-	if (search == NULL)
-		return FALSE;
-	if (app->id != NULL && strcasestr (app->id, search) != NULL)
-		return TRUE;
-	if (app->name != NULL && strcasestr (app->name, search) != NULL)
-		return TRUE;
-	if (app->summary != NULL && strcasestr (app->summary, search) != NULL)
-		return TRUE;
-	if (app->description != NULL && strcasestr (app->description, search) != NULL)
-		return TRUE;
+	appstream_app_add_tokens (app, app->id, "C", 100);
+	appstream_app_add_tokens (app, app->name, app->name_lang, 80);
+	appstream_app_add_tokens (app, app->summary, app->summary_lang, 60);
 	for (i = 0; i < app->keywords->len; i++) {
 		tmp = g_ptr_array_index (app->keywords, i);
-		if (strcasestr (tmp, search) != NULL)
-			return TRUE;
+		appstream_app_add_tokens (app, tmp, "C", 40);
 	}
+	appstream_app_add_tokens (app, app->description, app->description_lang, 20);
 	for (i = 0; i < app->mimetypes->len; i++) {
 		tmp = g_ptr_array_index (app->mimetypes, i);
-		if (strcasestr (tmp, search) != NULL)
-			return TRUE;
+		appstream_app_add_tokens (app, tmp, "C", 1);
 	}
-	return FALSE;
+}
+
+/**
+ * appstream_app_search_matches:
+ */
+guint
+appstream_app_search_matches (AppstreamApp *app, const gchar *search)
+{
+	AppstreamAppTokenItem *token_item;
+	guint i, j;
+
+	/* nothing to do */
+	if (search == NULL)
+		return 0;
+
+	/* ensure the token cache is created */
+	if (g_once_init_enter (&app->token_cache_valid)) {
+		appstream_app_create_token_cache (app);
+		g_once_init_leave (&app->token_cache_valid, TRUE);
+	}
+
+	/* find the search term */
+	for (i = 0; i < app->token_cache->len; i++) {
+		token_item = g_ptr_array_index (app->token_cache, i);
+		for (j = 0; token_item->values_utf8[j] != NULL; j++) {
+			if (g_str_has_prefix (token_item->values_utf8[j], search))
+				return token_item->score;
+		}
+		for (j = 0; token_item->values_ascii[j] != NULL; j++) {
+			if (g_str_has_prefix (token_item->values_utf8[j], search))
+				return token_item->score / 2;
+		}
+	}
+	return 0;
 }
