@@ -141,6 +141,8 @@ appstream_cache_app_id_kind_from_string (const gchar *id_kind)
 		return APPSTREAM_APP_ID_KIND_CODEC;
 	if (g_strcmp0 (id_kind, "webapp") == 0)
 		return APPSTREAM_APP_ID_KIND_WEBAPP;
+	if (g_strcmp0 (id_kind, "source") == 0)
+		return APPSTREAM_APP_ID_KIND_SOURCE;
 	return APPSTREAM_APP_ID_KIND_UNKNOWN;
 }
 
@@ -847,6 +849,7 @@ appstream_cache_parse_file (AppstreamCache *cache,
 			     APPSTREAM_CACHE_ERROR_FAILED,
 			     "cannot process file of type %s",
 			     content_type);
+		goto out;
 	}
 
 	/* add to array to maintain a ref for the lifetime of the AppstreamApp */
