@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -32,6 +32,7 @@
 
 struct GsPluginPrivate {
 	PkClient		*client;
+	GHashTable		*sources;
 };
 
 /**
@@ -49,12 +50,15 @@ gs_plugin_get_name (void)
 void
 gs_plugin_initialize (GsPlugin *plugin)
 {
-	/* create private area */
 	plugin->priv = GS_PLUGIN_GET_PRIVATE (GsPluginPrivate);
 	plugin->priv->client = pk_client_new ();
 	pk_client_set_background (plugin->priv->client, FALSE);
 	pk_client_set_interactive (plugin->priv->client, FALSE);
 	pk_client_set_cache_age (plugin->priv->client, G_MAXUINT);
+	plugin->priv->sources = g_hash_table_new_full (g_str_hash,
+						       g_str_equal,
+						       g_free,
+						       g_free);
 }
 
 /**
@@ -76,6 +80,7 @@ gs_plugin_get_deps (GsPlugin *plugin)
 void
 gs_plugin_destroy (GsPlugin *plugin)
 {
+	g_hash_table_unref (plugin->priv->sources);
 	g_object_unref (plugin->priv->client);
 }
 
@@ -111,15 +116,33 @@ gs_plugin_packagekit_progress_cb (PkProgress *progress,
 		gs_plugin_status_update (plugin, NULL, plugin_status);
 }
 
+/**
+ * gs_plugin_packagekit_set_origin:
+ **/
+static void
+gs_plugin_packagekit_set_origin (GsPlugin *plugin,
+				 GsApp *app,
+				 const gchar *id)
+{
+	const gchar *name;
+	name = g_hash_table_lookup (plugin->priv->sources, id);
+	if (name != NULL)
+		gs_app_set_origin (app, name);
+	else
+		gs_app_set_origin (app, id);
+}
+
 /**
  * gs_plugin_packagekit_resolve_packages_app:
  **/
 static void
-gs_plugin_packagekit_resolve_packages_app (GPtrArray *packages,
+gs_plugin_packagekit_resolve_packages_app (GsPlugin *plugin,
+					   GPtrArray *packages,
 					   GsApp *app)
 {
 	GPtrArray *sources;
 	PkPackage *package;
+	const gchar *data;
 	const gchar *pkgname;
 	gchar *tmp;
 	guint i, j;
@@ -140,6 +163,12 @@ gs_plugin_packagekit_resolve_packages_app (GPtrArray *packages,
 				switch (pk_package_get_info (package)) {
 				case GS_APP_STATE_INSTALLED:
 					number_installed++;
+					data = pk_package_get_data (package);
+					if (g_str_has_prefix (data, "installed:")) {
+						gs_plugin_packagekit_set_origin (plugin,
+										 app,
+										 data + 10);
+					}
 					break;
 				case GS_APP_STATE_AVAILABLE:
 					number_available++;
@@ -196,7 +225,7 @@ gs_plugin_packagekit_resolve_packages (GsPlugin *plugin,
 {
 	GList *l;
 	GPtrArray *array = NULL;
-	GPtrArray *package_ids;
+	GPtrArray *package_ids = NULL;
 	GPtrArray *packages = NULL;
 	GPtrArray *sources;
 	GsApp *app;
@@ -246,10 +275,11 @@ gs_plugin_packagekit_resolve_packages (GsPlugin *plugin,
 	packages = pk_results_get_package_array (results);
 	for (l = list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
-		gs_plugin_packagekit_resolve_packages_app (packages, app);
+		gs_plugin_packagekit_resolve_packages_app (plugin, packages, app);
 	}
 out:
-	g_ptr_array_unref (package_ids);
+	if (package_ids != NULL)
+		g_ptr_array_unref (package_ids);
 	if (packages != NULL)
 		g_ptr_array_unref (packages);
 	if (error_code != NULL)
@@ -574,6 +604,47 @@ out:
 	return ret;
 }
 
+/**
+ * gs_plugin_packagekit_get_source_list:
+ **/
+static gboolean
+gs_plugin_packagekit_get_source_list (GsPlugin *plugin,
+				      GCancellable *cancellable,
+				      GError **error)
+{
+	GPtrArray *array = NULL;
+	PkRepoDetail *rd;
+	PkResults *results;
+	gboolean ret = TRUE;
+	guint i;
+
+	/* ask PK for the repo details */
+	results = pk_client_get_repo_list (plugin->priv->client,
+					   pk_bitfield_from_enums (PK_FILTER_ENUM_NONE, -1),
+					   cancellable,
+					   gs_plugin_packagekit_progress_cb, plugin,
+					   error);
+	if (results == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+	array = pk_results_get_repo_detail_array (results);
+	for (i = 0; i < array->len; i++) {
+		rd = g_ptr_array_index (array, i);
+#if PK_CHECK_VERSION(0,9,1)
+		g_hash_table_insert (plugin->priv->sources,
+				     g_strdup (pk_repo_detail_get_id (rd)),
+				     g_strdup (pk_repo_detail_get_description (rd)));
+#endif
+	}
+out:
+	if (array != NULL)
+		g_ptr_array_unref (array);
+	if (results != NULL)
+		g_object_unref (results);
+	return ret;
+}
+
 /**
  * gs_plugin_refine_requires_version:
  */
@@ -624,6 +695,8 @@ gs_plugin_refine_requires_package_id (GsApp *app, GsPluginRefineFlags flags)
 		return TRUE;
 	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS) > 0)
 		return TRUE;
+	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN) > 0)
+		return TRUE;
 	return FALSE;
 }
 
@@ -645,6 +718,16 @@ gs_plugin_refine (GsPlugin *plugin,
 	const gchar *tmp;
 	gboolean ret = TRUE;
 
+	/* get the repo_id -> repo_name mapping set up */
+	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN) > 0 &&
+	     g_hash_table_size (plugin->priv->sources) == 0) {
+		ret = gs_plugin_packagekit_get_source_list (plugin,
+							    cancellable,
+							    error);
+		if (!ret)
+			goto out;
+	}
+
 	/* can we resolve in one go? */
 	gs_profile_start (plugin->profile, "packagekit-refine[name->id]");
 	for (l = *list; l != NULL; l = l->next) {
