@@ -166,6 +166,7 @@ out:
  *
  * The cases we have to deal with:
  *  - Single line text, so all to summary
+ *  - Single line long text, so all to description
  *  - Multiple line text, so first line to summary and the rest to description
  */
 static void
@@ -178,7 +179,11 @@ gs_plugin_packagekit_refresh_set_text (GsApp *app, const gchar *text)
 	tmp = g_strdup (text);
 	nl = g_strstr_len (tmp, -1, "\n");
 	if (nl == NULL) {
-		gs_app_set_summary (app, GS_APP_QUALITY_LOWEST, text);
+		if (strlen (text) < 40) {
+			gs_app_set_summary (app, GS_APP_QUALITY_LOWEST, text);
+			goto out;
+		}
+		gs_app_set_description (app, GS_APP_QUALITY_LOWEST, text);
 		goto out;
 	}
 	*nl = '\0';
@@ -205,16 +210,23 @@ gs_plugin_filename_to_app (GsPlugin *plugin,
 	GPtrArray *array = NULL;
 	GsApp *app = NULL;
 	PkDetails *item;
-	PkResults *results;
+	PkResults *results = NULL;
 
 	/* get details */
 	files = g_strsplit (filename, "\t", -1);
 	pk_client_set_cache_age (PK_CLIENT (plugin->priv->task), G_MAXUINT);
+#if PK_CHECK_VERSION(0,9,1)
 	results = pk_client_get_details_local (PK_CLIENT (plugin->priv->task),
 					       files,
 					       cancellable,
 					       gs_plugin_packagekit_progress_cb, plugin,
 					       error);
+#else
+	g_set_error_literal (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_FAILED,
+			     "GetDetailsLocal() not supported");
+#endif
 	if (results == NULL) {
 		ret = FALSE;
 		goto out;
